# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from .webservices_flags import (NFE_AMBIENTE_PRODUCAO,
                                NFE_AMBIENTE_HOMOLOGACAO,
                                WS_DPEC_CONSULTA,
                                WS_DPEC_RECEPCAO,
                                WS_NFE_CANCELAMENTO,
                                WS_NFE_CONSULTA,
                                WS_NFE_CONSULTA_CADASTRO,
                                WS_NFE_CONSULTA_RECIBO,
                                WS_NFE_ENVIO_LOTE,
                                WS_NFE_INUTILIZACAO,
                                WS_NFE_SITUACAO)


METODO_WS = {
    WS_NFE_ENVIO_LOTE: {
        'webservice': 'NfeRecepcao',
        'metodo'    : 'nfeRecepcaoLote',
    },
    WS_NFE_CONSULTA_RECIBO: {
        'webservice': 'NfeRetRecepcao',
        'metodo'    : 'nfeRetRecepcao',
    },
    WS_NFE_CANCELAMENTO: {
        'webservice': 'NfeCancelamento',
        'metodo'    : 'nfeCancelamentoNF',
    },
    WS_NFE_INUTILIZACAO: {
        'webservice': 'NfeInutilizacao',
        'metodo'    : 'nfeInutilizacaoNF',
    },
    WS_NFE_CONSULTA: {
        'webservice': 'NfeConsulta',
        'metodo'    : 'nfeConsultaNF',
    },
    WS_NFE_SITUACAO: {
        'webservice': 'NfeStatusServico',
        'metodo'    : 'nfeStatusServicoNF',
    },
    WS_NFE_CONSULTA_CADASTRO: {
        'webservice': 'CadConsultaCadastro',
        'metodo'    : 'consultaCadastro',
    }
}

SVRS = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'            : 'nfe.sefazvirtual.rs.gov.br',
        WS_NFE_ENVIO_LOTE        : 'ws/nferecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO: 'ws/nferetrecepcao/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO    : 'ws/nfecancelamento/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO    : 'ws/nfeinutilizacao/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA        : 'ws/nfeconsulta/NfeConsulta.asmx',
        WS_NFE_SITUACAO        : 'ws/nfestatusservico/NfeStatusServico.asmx'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'            : 'homologacao.nfe.sefazvirtual.rs.gov.br',
        WS_NFE_ENVIO_LOTE        : 'ws/nferecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO: 'ws/nferetrecepcao/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO    : 'ws/nfecancelamento/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO    : 'ws/nfeinutilizacao/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA        : 'ws/nfeconsulta/NfeConsulta.asmx',
        WS_NFE_SITUACAO        : 'ws/nfestatusservico/NfeStatusServico.asmx'
        }
}

SVAN = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'            : 'www.sefazvirtual.fazenda.gov.br',
        WS_NFE_ENVIO_LOTE        : 'NfeRecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO: 'NFeRetRecepcao/NFeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO    : 'NFeCancelamento/NFeCancelamento.asmx',
        WS_NFE_INUTILIZACAO    : 'NFeInutilizacao/NFeInutilizacao.asmx',
        WS_NFE_CONSULTA        : 'nfeconsulta/nfeconsulta.asmx',
        WS_NFE_SITUACAO        : 'NFeStatusServico/NFeStatusServico.asmx'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'            : 'hom.nfe.fazenda.gov.br',
        WS_NFE_ENVIO_LOTE        : 'NfeRecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO: 'NFeRetRecepcao/NFeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO    : 'NFeCancelamento/NFeCancelamento.asmx',
        WS_NFE_INUTILIZACAO    : 'NFeInutilizacao/NFeInutilizacao.asmx',
        WS_NFE_CONSULTA        : 'nfeconsulta/nfeconsulta.asmx',
        WS_NFE_SITUACAO        : 'NFeStatusServico/NFeStatusServico.asmx'
        }
}

SCAN = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'            : 'www.scan.fazenda.gov.br',
        WS_NFE_ENVIO_LOTE        : 'NfeRecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO: 'NfeRetRecepcao/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO    : 'NfeCancelamento/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO    : 'NfeInutilizacao/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA        : 'NfeConsulta/NfeConsulta.asmx',
        WS_NFE_SITUACAO        : 'NfeStatusServico/NfeStatusServico.asmx'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'            : 'hom.nfe.fazenda.gov.br',
        WS_NFE_ENVIO_LOTE        : 'SCAN/NfeRecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO: 'SCAN/NfeRetRecepcao/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO    : 'SCAN/NfeCancelamento/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO    : 'SCAN/NfeInutilizacao/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA        : 'SCAN/NfeConsulta/NfeConsulta.asmx',
        WS_NFE_SITUACAO        : 'SCAN/NfeStatusServico/NfeStatusServico.asmx'
        }
}

DPEC = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'     : 'www.nfe.fazenda.gov.br',
        WS_DPEC_CONSULTA: 'SCERecepcaoRFB/SCERecepcaoRFB.asmx',
        WS_DPEC_RECEPCAO: 'SCEConsultaRFB/SCEConsultaRFB.asmx'
    },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'     : 'hom.nfe.fazenda.gov.br',
        WS_DPEC_CONSULTA: 'SCERecepcaoRFB/SCERecepcaoRFB.asmx',
        WS_DPEC_RECEPCAO: 'SCEConsultaRFB/SCEConsultaRFB.asmx'
    }
}

UFAM = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'            : 'nfe.sefaz.am.gov.br',
        WS_NFE_ENVIO_LOTE        : 'ws/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO: 'ws/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO    : 'ws/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO    : 'ws/services/NfeInutilizacao',
        WS_NFE_CONSULTA        : 'ws/services/NfeConsulta',
        WS_NFE_SITUACAO        : 'ws/services/NfeStatusServico'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'            : 'homnfe.sefaz.am.gov.br',
        WS_NFE_ENVIO_LOTE        : 'ws/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO: 'ws/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO    : 'ws/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO    : 'ws/services/NfeInutilizacao',
        WS_NFE_CONSULTA        : 'ws/services/NfeConsulta',
        WS_NFE_SITUACAO        : 'ws/services/NfeStatusServico'
        }
}

UFBA = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.sefaz.ba.gov.br',
        WS_NFE_ENVIO_LOTE         : 'webservices/nfe/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO : 'webservices/nfe/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO     : 'webservices/nfe/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO     : 'webservices/nfe/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA         : 'webservices/nfe/NfeConsulta.asmx',
        WS_NFE_SITUACAO         : 'webservices/nfe/NfeStatusServico.asmx',
        WS_NFE_CONSULTA_CADASTRO: 'webservices/nfe/NfeConsulta.asmx'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'            : 'hnfe.sefaz.ba.gov.br',
        WS_NFE_ENVIO_LOTE        : 'webservices/nfe/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO: 'webservices/nfe/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO    : 'webservices/nfe/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO    : 'webservices/nfe/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA        : 'webservices/nfe/NfeConsulta.asmx',
        WS_NFE_SITUACAO        : 'webservices/nfe/NfeStatusServico.asmx'
        }
}

UFCE = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.sefaz.ce.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe/services/CadConsultaCadastro'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'nfeh.sefaz.ce.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe/services/CadConsultaCadastro'
        }
}

#UFDF = {
    #NFE_AMBIENTE_PRODUCAO: {
        #'servidor'             : 'dec.fazenda.df.gov.br',
        #WS_NFE_ENVIO_LOTE         : 'nfe/ServiceRecepcao.asmx',
        #WS_NFE_CONSULTA_RECIBO : 'nfe/ServiceRetRecepcao.asmx',
        #WS_NFE_CANCELAMENTO     : 'nfe/ServiceCancelamento.asmx',
        #WS_NFE_INUTILIZACAO     : 'nfe/ServiceInutilizacao.asmx',
        #WS_NFE_CONSULTA         : 'nfe/ServiceConsulta.asmx',
        #WS_NFE_SITUACAO         : 'nfe/ServiceStatus.asmx',
        #WS_NFE_CONSULTA_CADASTRO: 'nfe/ServiceConsultaCadastro.asmx',
        #},
    #NFE_AMBIENTE_HOMOLOGACAO: {
        #'servidor'             : 'homolog.nfe.fazenda.df.gov.br',
        #WS_NFE_ENVIO_LOTE         : 'nfe/ServiceRecepcao.asmx',
        #WS_NFE_CONSULTA_RECIBO : 'nfe/ServiceRetRecepcao.asmx',
        #WS_NFE_CANCELAMENTO     : 'nfe/ServiceCancelamento.asmx',
        #WS_NFE_INUTILIZACAO     : 'nfe/ServiceInutilizacao.asmx',
        #WS_NFE_CONSULTA         : 'nfe/ServiceConsulta.asmx',
        #WS_NFE_SITUACAO         : 'nfe/ServiceStatus.asmx',
        #WS_NFE_CONSULTA_CADASTRO: 'nfe/ServiceConsultaCadastro.asmx'
        #}
#}

UFGO = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.sefaz.go.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe/services/CadConsultaCadastro'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'homolog.sefaz.go.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe/services/CadConsultaCadastro'
        }
}

UFMT = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.sefaz.mt.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfews/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfews/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfews/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfews/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfews/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfews/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfews/CadConsultaCadastro'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'homologacao.sefaz.mt.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfews/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfews/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfews/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfews/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfews/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfews/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfews/CadConsultaCadastro'
        }
}

UFMS = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'producao.nfe.ms.gov.br',
        WS_NFE_ENVIO_LOTE         : 'producao/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'producao/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'producao/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'producao/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'producao/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'producao/services/NfeStatusServico'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'homologacao.nfe.ms.gov.br',
        WS_NFE_ENVIO_LOTE         : 'homologacao/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'homologacao/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'homologacao/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'homologacao/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'homologacao/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'homologacao/services/NfeStatusServico'
        }
}

UFMG = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.fazenda.mg.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe/services/CadConsultaCadastro'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'hnfe.fazenda.mg.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe/services/CadConsultaCadastro'
        }
}

UFPR = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.fazenda.pr.gov.br',
        WS_NFE_ENVIO_LOTE         : 'NFENWebServices/services/nfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'NFENWebServices/services/nfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'NFENWebServices/services/nfeCancelamentoNF',
        WS_NFE_INUTILIZACAO     : 'NFENWebServices/services/nfeInutilizacaoNF',
        WS_NFE_CONSULTA         : 'NFENWebServices/services/nfeConsultaNF',
        WS_NFE_SITUACAO         : 'NFENWebServices/services/nfeStatusServicoNF'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'homologacao.nfe.fazenda.pr.gov.br',
        WS_NFE_ENVIO_LOTE         : 'NFENWebServices/services/nfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'NFENWebServices/services/nfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'NFENWebServices/services/nfeCancelamentoNF',
        WS_NFE_INUTILIZACAO     : 'NFENWebServices/services/nfeInutilizacaoNF',
        WS_NFE_CONSULTA         : 'NFENWebServices/services/nfeConsultaNF',
        WS_NFE_SITUACAO         : 'NFENWebServices/services/nfeStatusServicoNF'
        }
}

UFPE = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.sefaz.pe.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe-service/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe-service/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe-service/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe-service/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe-service/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe-service/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe-service/services/CadConsultaCadastro'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'nfehomolog.sefaz.pe.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfe-service/services/NfeRecepcao',
        WS_NFE_CONSULTA_RECIBO : 'nfe-service/services/NfeRetRecepcao',
        WS_NFE_CANCELAMENTO     : 'nfe-service/services/NfeCancelamento',
        WS_NFE_INUTILIZACAO     : 'nfe-service/services/NfeInutilizacao',
        WS_NFE_CONSULTA         : 'nfe-service/services/NfeConsulta',
        WS_NFE_SITUACAO         : 'nfe-service/services/NfeStatusServico',
        WS_NFE_CONSULTA_CADASTRO: 'nfe-service/services/CadConsultaCadastro'
    }
}

UFRS = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.sefaz.rs.gov.br',
        WS_NFE_ENVIO_LOTE         : 'ws/nferecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO : 'ws/nferetrecepcao/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO     : 'ws/nfecancelamento/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO     : 'ws/nfeinutilizacao/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA         : 'ws/nfeconsulta/NfeConsulta.asmx',
        WS_NFE_SITUACAO         : 'ws/nfestatusservico/NfeStatusServico.asmx',
        WS_NFE_CONSULTA_CADASTRO: 'ws/CadConsultaCadastro/CadConsultaCadastro.asmx'
    },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'homologacao.nfe.sefaz.rs.gov.br',
        WS_NFE_ENVIO_LOTE         : 'ws/nferecepcao/NfeRecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO : 'ws/nferetrecepcao/NfeRetRecepcao.asmx',
        WS_NFE_CANCELAMENTO     : 'ws/nfecancelamento/NfeCancelamento.asmx',
        WS_NFE_INUTILIZACAO     : 'ws/nfeinutilizacao/NfeInutilizacao.asmx',
        WS_NFE_CONSULTA         : 'ws/nfeconsulta/NfeConsulta.asmx',
        WS_NFE_SITUACAO         : 'ws/nfestatusservico/NfeStatusServico.asmx'
    }
}

#UFRO = {
    #NFE_AMBIENTE_PRODUCAO: {
        #'servidor'             : 'ws.nfe.sefin.ro.gov.br',
        #WS_NFE_ENVIO_LOTE         : SVRS[NFE_AMBIENTE_PRODUCAO][WS_NFE_ENVIO_LOTE],
        #WS_NFE_CONSULTA_RECIBO : SVRS[NFE_AMBIENTE_PRODUCAO][WS_NFE_CONSULTA_RECIBO],
        #WS_NFE_CANCELAMENTO     : 'wsprod/NfeCancelamento',
        #WS_NFE_INUTILIZACAO     : SVRS[NFE_AMBIENTE_PRODUCAO][WS_NFE_INUTILIZACAO],
        #WS_NFE_CONSULTA         : 'wsprod/NfeConsulta',
        #WS_NFE_SITUACAO         : 'wsprod/NfeStatusServico',
        #WS_NFE_CONSULTA_CADASTRO: 'wsprod/CadConsultaCadastro'
    #},
    #NFE_AMBIENTE_HOMOLOGACAO: {
        #'servidor'             : 'ws.nfe.sefin.ro.gov.br',
        #WS_NFE_ENVIO_LOTE         : SVRS[NFE_AMBIENTE_HOMOLOGACAO][WS_NFE_ENVIO_LOTE],
        #WS_NFE_CONSULTA_RECIBO : SVRS[NFE_AMBIENTE_HOMOLOGACAO][WS_NFE_CONSULTA_RECIBO],
        #WS_NFE_CANCELAMENTO     : 'ws/NfeCancelamento',
        #WS_NFE_INUTILIZACAO     : SVRS[NFE_AMBIENTE_HOMOLOGACAO][WS_NFE_INUTILIZACAO],
        #WS_NFE_CONSULTA         : 'ws/NfeConsulta',
        #WS_NFE_SITUACAO         : 'ws/NfeStatusServico',
        #WS_NFE_CONSULTA_CADASTRO: 'ws/CadConsultaCadastro'
    #}
#}

UFSP = {
    NFE_AMBIENTE_PRODUCAO: {
        'servidor'             : 'nfe.fazenda.sp.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfeweb/services/nferecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO : 'nfeweb/services/nferetrecepcao.asmx',
        WS_NFE_CANCELAMENTO     : 'nfeweb/services/nfecancelamento.asmx',
        WS_NFE_INUTILIZACAO     : 'nfeweb/services/nfeinutilizacao.asmx',
        WS_NFE_CONSULTA         : 'nfeweb/services/nfeconsulta.asmx',
        WS_NFE_SITUACAO         : 'nfeweb/services/nfestatusservico.asmx',
        WS_NFE_CONSULTA_CADASTRO: 'nfeweb/services/cadconsultacadastro.asmx'
        },
    NFE_AMBIENTE_HOMOLOGACAO: {
        'servidor'             : 'homologacao.nfe.fazenda.sp.gov.br',
        WS_NFE_ENVIO_LOTE         : 'nfeweb/services/nferecepcao.asmx',
        WS_NFE_CONSULTA_RECIBO : 'nfeweb/services/nferetrecepcao.asmx',
        WS_NFE_CANCELAMENTO     : 'nfeweb/services/nfecancelamento.asmx',
        WS_NFE_INUTILIZACAO     : 'nfeweb/services/nfeinutilizacao.asmx',
        WS_NFE_CONSULTA         : 'nfeweb/services/nfeconsulta.asmx',
        WS_NFE_SITUACAO         : 'nfeweb/services/nfestatusservico.asmx',
        WS_NFE_CONSULTA_CADASTRO: 'nfeWEB/services/cadconsultacadastro.asmx'
        }
}


#
# Informação obtida em
# http://www.nfe.fazenda.gov.br/portal/VerificacaoDeServicos/VerificacaoServicos.aspx
#  Última verificação: 07/04/2010 16:30:14
#  * Estados Emissores pela Sefaz Virtual RS (Rio Grande do Sul): AC, AL, AM, AP, DF, MS, PB, RJ, RO, RR, SC, SE e TO.
#  ** Estados Emissores pela Sefaz Virtual AN (Ambiente Nacional): CE, ES, MA, PA, PI e RN.
# Estados que têm seus próprios servidores: BA, GO, MG, MT, PE, PR, SP.
#


ESTADO_WS = {
    'AC': SVRS,
    'AL': SVRS,
    'AM': SVRS,
    'AP': SVRS,
    'BA': UFBA,
    'CE': SVAN,
    'DF': SVRS,
    'ES': SVAN,
    'GO': UFGO,
    'MA': SVAN,
    'MG': UFMG,
    'MS': SVRS,
    'MT': UFMT,
    'PA': SVAN,
    'PB': SVRS,
    'PE': UFPE,
    'PI': SVAN,
    'PR': UFPR,
    'RJ': SVRS,
    'RN': SVAN,
    'RO': SVRS,
    'RR': SVRS,
    'RS': UFRS,
    'SC': SVRS,
    'SE': SVRS,
    'SP': UFSP,
    'TO': SVRS
}



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import NAMESPACE_NFE, TagDecimal
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_2 as ESQUEMA_ATUAL
from pysped.nfe.leiaute import consstatserv_107
import os


DIRNAME = os.path.dirname(__file__)


class ConsStatServ(consstatserv_107.ConsStatServ):
    def __init__(self):
        super(ConsStatServ, self).__init__()
        self.versao = TagDecimal(nome='consStatServ', codigo='FP01', propriedade='versao', namespace=NAMESPACE_NFE, valor='2.00', raiz='/')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consStatServ_v2.00.xsd'

class RetConsStatServ(consstatserv_107.RetConsStatServ):
    def __init__(self):
        super(RetConsStatServ, self).__init__()
        self.versao    = TagDecimal(nome='retConsStatServ', codigo='FR01', propriedade='versao', namespace=NAMESPACE_NFE, valor='2.00', raiz='/')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retConsStatServ_v2.00.xsd'



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from reportlab.lib.units import cm
from reportlab.lib.pagesizes import A4, landscape
from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT
from reportlab.lib.colors import HexColor

from geraldo import ReportBand
from geraldo import ObjectValue, Label
import os


DIRNAME = os.path.dirname(__file__)


''' Margens e tamanhos padronizados '''
RETRATO = A4
PAISAGEM = landscape(A4)
MARGEM_SUPERIOR = 0.8*cm
MARGEM_INFERIOR = 0.8*cm
MARGEM_ESQUERDA = 0.8*cm
MARGEM_DIREITA = 0.8*cm
LARGURA_RETRATO = RETRATO[0] - MARGEM_ESQUERDA - MARGEM_DIREITA
LARGURA_PAISAGEM = PAISAGEM[0] - MARGEM_ESQUERDA - MARGEM_DIREITA


#
# Fontes adicionais
#
FONTES_ADICIONAIS = {
    u'Gentium Book Basic': (
        (u'Gentium Book Basic'            , DIRNAME + u'/fonts/genbkbasr.ttf' , False, False),
        (u'Gentium Book Basic Bold'       , DIRNAME + u'/fonts/genbkbasb.ttf' , True , False),
        (u'Gentium Book Basic Italic'     , DIRNAME + u'/fonts/genbkbasi.ttf' , False, True),
        (u'Gentium Book Basic Bold Italic', DIRNAME + u'/fonts/genbkbasbi.ttf', True , True),
    )
}

#
# Estilos padronizados
#
FONTE_NORMAL = 'Gentium Book Basic'
FONTE_NEGRITO = FONTE_NORMAL + ' Bold'
FONTE_ITALICO = FONTE_NORMAL + ' Italic'
FONTE_NEGRITO_ITALICO = FONTE_NORMAL + ' Bold Italic'

FONTE_TAMANHO_5 = 5
FONTE_TAMANHO_6 = FONTE_TAMANHO_5 + 1
FONTE_TAMANHO_7 = FONTE_TAMANHO_5 + 2
FONTE_TAMANHO_8 = FONTE_TAMANHO_5 + 3
FONTE_TAMANHO_85 = FONTE_TAMANHO_5 + 3.5
FONTE_TAMANHO_9 = FONTE_TAMANHO_5 + 4
FONTE_TAMANHO_10 = FONTE_TAMANHO_5 * 2
FONTE_TAMANHO_11 = FONTE_TAMANHO_10 + 1
FONTE_TAMANHO_12 = FONTE_TAMANHO_10 + 2
FONTE_TAMANHO_14 = FONTE_TAMANHO_10 + 4
FONTE_TAMANHO_18 = FONTE_TAMANHO_10 + 8
FONTE_TAMANHO_40 = FONTE_TAMANHO_10 * 4

VERMELHO_CARIMBO = HexColor(0xff9393)
CINZA_MARCADAGUA = HexColor(0x939393)

DESCRITIVO_BLOCO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_8}
DESCRITIVO_CAMPO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_5}
DESCRITIVO_CAMPO_NEGRITO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_5}
DESCRITIVO_PRODUTO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_5, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_7}

DADO_CHAVE = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_85, 'alignment': TA_CENTER}
DADO_VARIAVEL = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_9, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_11}

DADO_CAMPO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_10, 'leading': FONTE_TAMANHO_12}
DADO_CAMPO_NEGRITO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_10, 'leading': FONTE_TAMANHO_12}
DADO_CAMPO_NUMERICO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_10, 'alignment': TA_RIGHT, 'leading': FONTE_TAMANHO_12}
DADO_CAMPO_NUMERICO_NEGRITO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_10, 'alignment': TA_RIGHT, 'leading': FONTE_TAMANHO_12}

DADO_PRODUTO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_6, 'leading': FONTE_TAMANHO_8}
DADO_PRODUTO_NUMERICO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_6, 'alignment': TA_RIGHT, 'leading': FONTE_TAMANHO_8}
DADO_PRODUTO_CENTRALIZADO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_6, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_8}

DADO_COMPLEMENTAR = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_6, 'leading': FONTE_TAMANHO_8}

DESCRITIVO_DANFE = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_12, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_12}
DESCRITIVO_NUMERO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_10, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_10}
DESCRITIVO_DANFE_GERAL = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_7, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_7}
DESCRITIVO_DANFE_ES = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_7, 'alignment': TA_LEFT, 'leading': FONTE_TAMANHO_7}

OBS_CONTINGENCIA = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_18, 'alignment': TA_CENTER, 'textColor': CINZA_MARCADAGUA}
OBS_HOMOLOGACAO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_40, 'alignment': TA_CENTER, 'textColor': VERMELHO_CARIMBO}
OBS_CANCELAMENTO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_40, 'leading': FONTE_TAMANHO_40+24, 'alignment': TA_CENTER, 'textColor': VERMELHO_CARIMBO, 'borderWidth': 3, 'borderColor': VERMELHO_CARIMBO, 'borderRadius': 3}
OBS_DENEGACAO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_40, 'leading': FONTE_TAMANHO_40+36, 'alignment': TA_CENTER, 'textColor': VERMELHO_CARIMBO, 'borderWidth': 3, 'borderColor': VERMELHO_CARIMBO, 'borderRadius': 3}
DESCRITIVO_CAMPO_CANCELAMENTO = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_5, 'leading': FONTE_TAMANHO_5, 'textColor': VERMELHO_CARIMBO, 'backColor': 'white'}
DADO_VARIAVEL_CANCELAMENTO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_9, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_11, 'textColor': VERMELHO_CARIMBO}

DADO_IMPRESSAO = {'fontName': FONTE_NORMAL, 'fontSize': FONTE_TAMANHO_5, 'leading': FONTE_TAMANHO_7}

EMIT_NOME  = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_12, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_14}
EMIT_DADOS = {'fontName': FONTE_NEGRITO, 'fontSize': FONTE_TAMANHO_8, 'alignment': TA_CENTER, 'leading': FONTE_TAMANHO_10}


class LabelMargemEsquerda(Label):
    def __init__(self):
        super(LabelMargemEsquerda, self).__init__()
        #self.borders_stroke_width = {'top': 0.1, 'right': 0.1, 'bottom': 0.1, 'left': 0.1}
        self.borders = {'top': 0.1, 'right': 0.1, 'bottom': 0.1, 'left': False}
        self.padding_top = 0.08*cm
        self.padding_left = 0.08*cm
        self.padding_bottom = 0.08*cm
        self.padding_right = 0.08*cm
        self.style = DESCRITIVO_CAMPO
        self.height = 0.70*cm


class LabelMargemDireita(LabelMargemEsquerda):
    def __init__(self):
        super(LabelMargemDireita, self).__init__()
        self.borders = {'top': 0.1, 'right': False, 'bottom': 0.1, 'left': 0.1}


class Campo(ObjectValue):
    def __init__(self):
        super(Campo, self).__init__()
        self.padding_top = 0.1*cm
        self.padding_left = 0.1*cm
        self.padding_bottom = 0.1*cm
        self.padding_right = 0.1*cm
        self.style = DADO_CAMPO
        self.height = 0.70*cm


class Texto(Label):
    def __init__(self):
        super(Texto, self).__init__()
        self.padding_top = 0.1*cm
        self.padding_left = 0.1*cm
        self.padding_bottom = 0.1*cm
        self.padding_right = 0.1*cm
        self.style = DADO_CAMPO
        self.height = 0.70*cm


class Descritivo(Label):
    def __init__(self):
        super(Descritivo, self).__init__()
        #self.borders_stroke_width = {'top': 0.1, 'right': 0.1, 'bottom': 0.1, 'left': 0.1}
        self.borders = {'top': 0.1, 'right': False, 'bottom': 0.1, 'left': False}
        self.padding_top = 0.03*cm
        self.padding_left = 0.1*cm
        #self.padding_bottom = 0.05*cm
        self.padding_right = 0.1*cm
        self.style = DESCRITIVO_BLOCO
        self.height = 0.42*cm


class BandaDANFE(ReportBand):
    def __init__(self):
        super(BandaDANFE, self).__init__()

    def _inclui_titulo(self, nome, titulo, top, left, width, height=None, margem_direita=False):
        # Prepara o Label com o título
        if margem_direita:
            lbl = LabelMargemDireita()
        else:
            lbl = LabelMargemEsquerda()

        lbl.name = 'lbl_' + nome
        lbl.text = titulo
        lbl.top = top
        lbl.left = left
        lbl.width = width

        if height:
            lbl.height = height

        return lbl

    def _inclui_campo(self, nome, conteudo, top, left, width, height=None):
        fld = Campo()
        fld.name = 'fld_' + nome
        fld.attribute_name = conteudo
        fld.top = top
        fld.left = left
        fld.width = width

        if height:
            fld.height = height

        return fld

    def _inclui_texto(self, nome, texto, top, left, width, height=None):
        lbl = Texto()
        lbl.name = 'txt_' + nome
        lbl.text = texto
        lbl.top = top
        lbl.left = left
        lbl.width = width

        if height:
            lbl.height = height

        return lbl

    def inclui_campo(self, nome, titulo, conteudo, top, left, width, height=None, margem_direita=False):
        lbl = self._inclui_titulo(nome, titulo, top, left, width, height, margem_direita)
        self.elements.append(lbl)

        fld = self._inclui_campo(nome, conteudo, top, left, width, height)
        fld.padding_top = 0.25*cm
        self.elements.append(fld)

        return lbl, fld

    def inclui_campo_numerico(self, nome, titulo, conteudo, top, left, width, height=None, margem_direita=False):
        lbl, fld = self.inclui_campo(nome, titulo, conteudo, top, left, width, height, margem_direita)
        fld.style = DADO_CAMPO_NUMERICO

        return lbl, fld

    def inclui_texto(self, nome, titulo, texto, top, left, width, height=None, margem_direita=False):
        lbl = self._inclui_titulo(nome, titulo, top, left, width, height, margem_direita)
        self.elements.append(lbl)

        if texto:
            txt = self._inclui_texto(nome, texto, top, left, width, height)
            txt.padding_top = 0.25*cm
            self.elements.append(txt)
        else:
            txt = None

        return lbl, txt

    def inclui_texto_numerico(self, nome, titulo, texto, top, left, width, height=None, margem_direita=False):
        lbl, txt = self.inclui_texto(nome, titulo, texto, top, left, width, height, margem_direita)

        if txt:
            txt.style = DADO_CAMPO_NUMERICO

        return lbl, txt

    def inclui_descritivo(self, nome, titulo, top, left, width, height=None):
        lbl = Descritivo()

        lbl.name = 'dsc_' + nome
        lbl.text = titulo
        lbl.top = top
        lbl.left = left
        lbl.width = width

        if height:
            lbl.height = height

        self.elements.append(lbl)

        return lbl

    def inclui_texto_sem_borda(self, nome, texto, top, left, width, height=None, margem_direita=False):
        txt = self._inclui_texto(nome, texto, top, left, width, height)
        txt.padding_top = 0.1*cm
        self.elements.append(txt)

        return txt

    def inclui_campo_sem_borda(self, nome, conteudo, top, left, width, height=None, margem_direita=False):
        fld = self._inclui_campo(nome, conteudo, top, left, width, height)
        fld.padding_top = 0.1*cm
        self.elements.append(fld)

        return fld

    def inclui_descritivo_produto(self, nome, titulo, top, left, width, height=None, margem_direita=False):
        lbl = self._inclui_titulo(nome, titulo, top, left, width, height, margem_direita)
        lbl.style = DESCRITIVO_PRODUTO
        lbl.padding_top = 0.05*cm
        lbl.padding_left = 0.05*cm
        lbl.padding_bottom = 0.05*cm
        lbl.padding_right = 0.05*cm

        if height:
            lbl.height = height
        else:
            lbl.height = 0.52*cm

        self.elements.append(lbl)
        return lbl

    def inclui_campo_produto(self, nome, conteudo, top, left, width, height=None, margem_direita=False):
        fld = self._inclui_campo(nome, conteudo, top, left, width, height)

        if margem_direita:
            fld.borders = {'top': 0.1, 'right': False, 'bottom': 0.1, 'left': 0.1}
        else:
            fld.borders = {'top': 0.1, 'right': 0.1, 'bottom': 0.1, 'left': False}

        fld.style = DADO_PRODUTO
        fld.padding_top = 0.05*cm
        fld.padding_left = 0.05*cm
        fld.padding_bottom = 0.05*cm
        fld.padding_right = 0.05*cm
        fld.auto_expand_height = True

        if height:
            fld.height = height
        else:
            fld.height = 0.28*cm

        self.elements.append(fld)

        return fld

    def inclui_campo_numerico_produto(self, nome, conteudo, top, left, width, height=None, margem_direita=False):
        fld = self.inclui_campo_produto(nome, conteudo, top, left, width, height, margem_direita)

        fld.style = DADO_PRODUTO_NUMERICO

        return fld

    def inclui_campo_centralizado_produto(self, nome, conteudo, top, left, width, height=None, margem_direita=False):
        fld = self.inclui_campo_produto(nome, conteudo, top, left, width, height, margem_direita)

        fld.style = DADO_PRODUTO_CENTRALIZADO

        return fld

    def inclui_texto_produto(self, nome, texto, top, left, width, height=None, margem_direita=False):
        txt = self._inclui_texto(nome, texto, top, left, width, height)
        txt.borders_stroke_width = {'top': 0.1, 'right': 0.1, 'bottom': 0.1, 'left': 0.1}

        if margem_direita:
            txt.borders = {'top': 0.1, 'right': False, 'bottom': 0.1, 'left': 0.1}
        else:
            txt.borders = {'top': 0.1, 'right': 0.1, 'bottom': 0.1, 'left': False}

        txt.style = DADO_PRODUTO
        txt.padding_top = 0.05*cm
        txt.padding_left = 0.05*cm
        txt.padding_bottom = 0.05*cm
        txt.padding_right = 0.05*cm
        txt.auto_expand_height = True

        if height:
            txt.height = height
        else:
            txt.height = 0.28*cm

        self.elements.append(txt)

        return txt

    def inclui_texto_numerico_produto(self, nome, texto, top, left, width, height=None, margem_direita=False):
        txt = self.inclui_texto_produto(nome, texto, top, left, width, height, margem_direita)

        txt.style = DADO_PRODUTO_NUMERICO

        return txt

    def inclui_texto_centralizado_produto(self, nome, texto, top, left, width, height=None, margem_direita=False):
        txt = self.inclui_texto_produto(nome, texto, top, left, width, height, margem_direita)

        txt.style = DADO_PRODUTO_CENTRALIZADO

        return txt



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Affero General Public License,
# publicada pela Free Software Foundation, em sua versão 3 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Affero General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Affero General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import *
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_3 as ESQUEMA_ATUAL
from pysped.nfe.leiaute import nfe_310
import os

DIRNAME = os.path.dirname(__file__)


class Deduc(nfe_310.Deduc):
    def __init__(self):
        super(Deduc, self).__init__()


class ForDia(nfe_310.ForDia):
    def __init__(self):
        super(ForDia, self).__init__()


class Cana(nfe_310.Cana):
    def __init__(self):
        super(Cana, self).__init__()


class IPIDevol(nfe_310.IPÎDevol):
    def __init__(self):
        super(IPIDevol, self).__init__()


class ImpostoDevol(nfe_310.ImpostoDevol):
    def __init__(self):
        super(ImpostoDevol, self).__init__()


class ISSQN(nfe_310.ISSQN):
    def __init__(self):
        super(ISSQN, self).__init__()


class ICMSUFDest(nfe_310.ICMSUFDest):
    def __init__(self):
        super(ICMSUFDest, self).__init__()


class COFINSST(nfe_310.COFINSST):
    def __init__(self):
        super(COFINSST, self).__init__()


class TagCSTCOFINS(nfe_310.TagCSTCOFINS):
    def __init__(self, *args, **kwargs):
        super(TagCSTCOFINS, self).__init__(*args, **kwargs)


class COFINS(nfe_310.COFINS):
    def __init__(self):
        super(COFINS, self).__init__()


class PISST(nfe_310.PISST):
    def __init__(self):
        super(PISST, self).__init__()


class TagCSTPIS(nfe_310.TagCSTPIS):
    def __init__(self, *args, **kwargs):
        super(TagCSTPIS, self).__init__(*args, **kwargs)


class PIS(nfe_310.PIS):
    def __init__(self):
        super(PIS, self).__init__()
        self.pPIS      = TagDecimal(nome='pPIS'     , codigo='Q08', tamanho=[1,  5, 1], decimais=[0, 4, 4], raiz='')


class II(nfe_310.II):
    def __init__(self):
        super(II, self).__init__()


class TagCSTIPI(nfe_310.TagCSTIPI):
    def __init__(self, *args, **kwargs):
        super(TagCSTIPI, self).__init__(*args, **kwargs)


class IPI(nfe_310.IPI):
    def __init__(self):
        super(IPI, self).__init__()


class TagCSOSN(nfe_310.TagCSOSN):
    def __init__(self, *args, **kwargs):
        super(TagCSOSN, self).__init__(*args, **kwargs)


class TagCSTICMS(nfe_310.TagCSTICMS):
    def __init__(self, *args, **kwargs):
        super(TagCSTICMS, self).__init__(*args, **kwargs)


class ICMS(nfe_310.ICMS):
    def __init__(self):
        super(ICMS, self).__init__()


class Imposto(nfe_310.Imposto):
    def __init__(self):
        super(Imposto, self).__init__()


class CIDE(nfe_310.CIDE):
    def __init__(self):
        super(CIDE, self).__init__()


class Comb(nfe_310.Comb):
    def __init__(self):
        super(Comb, self).__init__()


class Arma(nfe_310.Arma):
    def __init__(self):
        super(Arma, self).__init__()


class Med(nfe_310.Med):
    def __init__(self):
        super(Med, self).__init__()


class VeicProd(nfe_310.VeicProd):
    def __init__(self):
        super(VeicProd, self).__init__()


class ExportInd(nfe_310.ExportInd):
    def __init__(self):
        super(ExportInd, self).__init__()


class DetExport(nfe_310.DetExport):
    def __init__(self):
        super(DetExport, self).__init__()


class Adi(nfe_310.Adi):
    def __init__(self):
        super(Adi, self).__init__()


class DI(nfe_310.DI):
    def __init__(self):
        super(DI, self).__init__()


class Prod(nfe_310.Prod):
    def __init__(self):
        super(Prod, self).__init__()


class Det(nfe_310.Det):
    def __init__(self):
        super(Det, self).__init__()


class Compra(nfe_310.Compra):
    def __init__(self):
        super(Compra, self).__init__()


class Exporta(nfe_310.Exporta):
    def __init__(self):
        super(Exporta, self).__init__()


class ProcRef(nfe_310.ProcRef):
    def __init__(self):
        super(ProcRef, self).__init__()


class ObsFisco(nfe_310.ObsFisco):
    def __init__(self):
        super(ObsFisco, self).__init__()


class ObsCont(nfe_310.ObsCont):
    def __init__(self):
        super(ObsCont, self).__init__()


class InfAdic(nfe_310.InfAdic):
    def __init__(self):
        super(InfAdic, self).__init__()


class Card(nfe_310.Card):
    def __init__(self):
        super(Card, self).__init__()


class Pag(nfe_310.Pag):
    def __init__(self):
        super(Pag, self).__init__()


class Dup(nfe_310.Dup):
    def __init__(self):
        super(Dup, self).__init__()


class Fat(nfe_310.Fat):
    def __init__(self):
        super(Fat, self).__init__()


class Cobr(nfe_310.Cobr):
    def __init__(self):
        super(Cobr, self).__init__()


class Lacres(nfe_310.Lacres):
    def __init__(self):
        super(Lacres, self).__init__()


class Vol(nfe_310.Vol):
    def __init__(self, xml=None):
        super(Vol, self).__init__()


class Reboque(nfe_310.Reboque):
    def __init__(self):
        super(Reboque, self).__init__()


class VeicTransp(nfe_310.VeicTransp):
    def __init__(self):
        super(VeicTransp, self).__init__()


class RetTransp(nfe_310.RetTransp):
    def __init__(self):
        super(RetTransp, self).__init__()


class Transporta(nfe_310.Transporta):
    def __init__(self):
        super(Transporta, self).__init__()


class Transp(nfe_310.Transp):
    def __init__(self):
        super(Transp, self).__init__()


class RetTrib(nfe_310.RetTrib):
    def __init__(self):
        super(RetTrib, self).__init__()


class ISSQNTot(nfe_310.ISSQNTot):
    def __init__(self):
        super(ISSQNTot, self).__init__()


class ICMSTot(nfe_310.ICMSTot):
    def __init__(self):
        super(ICMSTot, self).__init__()


class Total(nfe_310.Total):
    def __init__(self):
        super(Total, self).__init__()


class AutXML(nfe_310.AutXML):
    def __init__(self):
        super(AutXML, self).__init__()


class Entrega(nfe_310.Entrega):
    def __init__(self):
        super(Entrega, self).__init__()


class Retirada(nfe_310.Retirada):
    def __init__(self):
        super(Retirada, self).__init__()


class EnderDest(nfe_310.EnderDest):
    def __init__(self):
        super(EnderDest, self).__init__()


class Dest(nfe_310.Dest):
    def __init__(self):
        super(Dest, self).__init__()


class Avulsa(nfe_310.Avulsa):
    def __init__(self):
        super(Avulsa, self).__init__()


class EnderEmit(nfe_310.EnderEmit):
    def __init__(self):
        super(EnderEmit, self).__init__()


class Emit(nfe_310.Emit):
    def __init__(self):
        super(Emit, self).__init__()


class RefECF(nfe_310.RefECF):
    def __init__(self):
        super(RefECF, self).__init__()


class RefNFP(nfe_310.RefNFP):
    def __init__(self):
        super(RefNFP, self).__init__()


class RefNF(nfe_310.RefNF):
    def __init__(self):
        super(RefNF, self).__init__()


class NFRef(nfe_310.NFRef):
    def __init__(self):
        super(NFRef, self).__init__()


class Ide(nfe_310.Ide):
    def __init__(self):
        super(Ide, self).__init__()


class InfNFe(nfe_310.InfNFe):
    def __init__(self):
        super(InfNFe, self).__init__()


class NFe(nfe_310.NFe):
    def __init__(self):
        super(NFe, self).__init__()



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from StringIO import StringIO
from geraldo.generators import PDFGenerator
from pysped.nfe.danfe.danferetrato import DANFERetrato
from pysped.nfe.leiaute import ProtNFe_310, RetCancNFe_200, ProcCancNFe_200
from pysped.nfe.leiaute import ProcEventoCancNFe_100


class DANFE(object):
    def __init__(self):
        self.imprime_canhoto        = True
        self.imprime_local_retirada = True
        self.imprime_local_entrega  = True
        self.imprime_fatura         = True
        self.imprime_duplicatas     = True
        self.imprime_issqn          = True

        self.caminho           = ''
        self.salvar_arquivo    = True

        self.NFe         = None
        self.protNFe     = None
        self.procCancNFe = None
        self.retCancNFe  = None
        self.procEventoCancNFe = None
        self.danfe       = None
        self.conteudo_pdf = None

        self.obs_impressao    = 'DANFE gerado em %(now:%d/%m/%Y, %H:%M:%S)s'
        self.nome_sistema     = ''
        self.site             = ''
        self.logo             = ''
        self.leiaute_logo_vertical = False
        self.dados_emitente   = []

    def gerar_danfe(self):
        if self.NFe is None:
            raise ValueError('Não é possível gerar um DANFE sem a informação de uma NF-e')

        if self.protNFe is None:
            self.protNFe = ProtNFe_310()

        if self.retCancNFe is None:
            self.retCancNFe = RetCancNFe_200()

        if self.procCancNFe is None:
            self.procCancNFe = ProcCancNFe_200()

        if self.procEventoCancNFe is None:
            self.procEventoCancNFe = ProcEventoCancNFe_100()

        #
        # Prepara o queryset para impressão
        #
        self.NFe.monta_chave()
        self.NFe.monta_dados_contingencia_fsda()
        self.NFe.site = self.site

        for detalhe in self.NFe.infNFe.det:
            detalhe.NFe = self.NFe
            detalhe.protNFe = self.protNFe
            detalhe.retCancNFe = self.retCancNFe
            detalhe.procCancNFe = self.procCancNFe
            detalhe.procEventoCancNFe = self.procEventoCancNFe

        #
        # Prepara as bandas de impressão para cada formato
        #
        if self.NFe.infNFe.ide.tpImp.valor == 2:
            raise ValueError('DANFE em formato paisagem ainda não implementado')
        else:
            self.danfe = DANFERetrato()
            self.danfe.queryset = self.NFe.infNFe.det

        if self.imprime_canhoto:
            self.danfe.band_page_header = self.danfe.canhoto
            self.danfe.band_page_header.child_bands = []
            self.danfe.band_page_header.child_bands.append(self.danfe.remetente)
        else:
            self.danfe.band_page_header = self.danfe.remetente
            self.danfe.band_page_header.child_bands = []

        # Emissão para simples conferência / sem protocolo de autorização
        if not self.protNFe.infProt.nProt.valor:
            self.danfe.remetente.campo_variavel_conferencia()

        # NF-e denegada
        elif self.protNFe.infProt.cStat.valor in ('110', '301', '302'):
            #self.danfe.remetente.campo_variavel_denegacao()
            self.danfe.remetente.campo_variavel_normal()
            self.danfe.remetente.obs_denegacao()

            #
            # Adiciona a observação de quem é a irregularidade fiscal
            #
            #if self.protNFe.infProt.cStat.valor == '301':
                #self.danfe.remetente.find_by_name('txt_remetente_var1').text = b'A circulação da mercadoria foi <font color="red"><b>PROIBIDA</b></font> pela SEFAZ<br />autorizadora, devido a irregularidades fiscais do emitente.'
            #elif self.protNFe.infProt.cStat.valor == '302':
                #self.danfe.remetente.find_by_name('txt_remetente_var1').text = b'A circulação da mercadoria foi <font color="red"><b>PROIBIDA</b></font> pela SEFAZ<br />autorizadora, devido a irregularidades fiscais do destinatário.'

        # Emissão em contingência com FS ou FSDA
        elif self.NFe.infNFe.ide.tpEmis.valor in (2, 5,):
            self.danfe.remetente.campo_variavel_contingencia_fsda()
            self.danfe.remetente.obs_contingencia_normal_scan()

        # Emissão em contingência com DPEC
        elif self.NFe.infNFe.ide.tpEmis.valor == 4:
            self.danfe.remetente.campo_variavel_contingencia_dpec()
            self.danfe.remetente.obs_contingencia_dpec()

        # Emissão normal ou contingência SCAN
        else:
            self.danfe.remetente.campo_variavel_normal()
            # Contingência SCAN
            if self.NFe.infNFe.ide.tpEmis.valor == 3:
                self.danfe.remetente.obs_contingencia_normal_scan()

        # A NF-e foi cancelada, no DANFE imprimir o "carimbo" de cancelamento
        if self.retCancNFe.infCanc.nProt.valor or self.procCancNFe.retCancNFe.infCanc.nProt.valor:
            if self.procCancNFe.cancNFe.infCanc.xJust.valor:
                self.danfe.remetente.obs_cancelamento_com_motivo()
            else:
                self.danfe.remetente.obs_cancelamento()

        # A NF-e foi cancelada por um evento de cancelamento, , no DANFE imprimir o "carimbo" de cancelamento
        if self.procEventoCancNFe.retEvento.infEvento.nProt.valor:
            if self.procEventoCancNFe.evento.infEvento.detEvento.xJust.valor:
              self.danfe.remetente.obs_cancelamento_com_motivo_evento()
            else:
              self.danfe.remetente.obs_cancelamento_evento()

        # Observação de ausência de valor fiscal
        # se não houver protocolo ou se o ambiente for de homologação
        if (not self.protNFe.infProt.nProt.valor) or self.NFe.infNFe.ide.tpAmb.valor == 2:
            self.danfe.remetente.obs_sem_valor_fiscal()

        self.danfe.band_page_header.child_bands.append(self.danfe.destinatario)

        if self.imprime_local_retirada and len(self.NFe.infNFe.retirada.xml):
            self.danfe.band_page_header.child_bands.append(self.danfe.local_retirada)

        if self.imprime_local_entrega and len(self.NFe.infNFe.entrega.xml):
            self.danfe.band_page_header.child_bands.append(self.danfe.local_entrega)

        if self.imprime_fatura:
            # Pagamento a prazo
            if (self.NFe.infNFe.ide.indPag.valor == 1) or \
                (len(self.NFe.infNFe.cobr.dup) > 1) or \
                ((len(self.NFe.infNFe.cobr.dup) == 1) and \
                (self.NFe.infNFe.cobr.dup[0].dVenc.valor.toordinal() > self.NFe.infNFe.ide.dEmi.valor.toordinal())):

                if self.imprime_duplicatas:
                    self.danfe.fatura_a_prazo.elements.append(self.danfe.duplicatas)

                self.danfe.band_page_header.child_bands.append(self.danfe.fatura_a_prazo)

            # Pagamento a vista
            elif (self.NFe.infNFe.ide.indPag.valor != 2):
                self.danfe.band_page_header.child_bands.append(self.danfe.fatura_a_vista)

                if self.imprime_duplicatas:
                    self.danfe.fatura_a_vista.elements.append(self.danfe.duplicatas)

        self.danfe.band_page_header.child_bands.append(self.danfe.calculo_imposto)
        self.danfe.band_page_header.child_bands.append(self.danfe.transporte)
        self.danfe.band_page_header.child_bands.append(self.danfe.cab_produto)

        if self.imprime_issqn and len(self.NFe.infNFe.total.ISSQNTot.xml):
            self.danfe.band_page_footer = self.danfe.iss
        else:
            self.danfe.band_page_footer = self.danfe.dados_adicionais

        self.danfe.band_detail = self.danfe.det_produto

        #
        # Observação de impressão
        #
        if self.nome_sistema:
            self.danfe.ObsImpressao.expression = self.nome_sistema + ' - ' + self.obs_impressao
        else:
            self.danfe.ObsImpressao.expression = self.obs_impressao

        #
        # Quadro do emitente
        #
        # Personalizado?
        if self.dados_emitente:
            self.danfe.remetente.monta_quadro_emitente(self.dados_emitente)
        else:
            # Sem logotipo
            if not self.logo:
                self.danfe.remetente.monta_quadro_emitente(self.danfe.remetente.dados_emitente_sem_logo())

            # Logotipo na vertical
            elif self.leiaute_logo_vertical:
                self.danfe.remetente.monta_quadro_emitente(self.danfe.remetente.dados_emitente_logo_vertical(self.logo))

            # Logotipo na horizontal
            else:
                self.danfe.remetente.monta_quadro_emitente(self.danfe.remetente.dados_emitente_logo_horizontal(self.logo))

        danfe_pdf = StringIO()
        self.danfe.generate_by(PDFGenerator, filename=danfe_pdf)
        self.conteudo_pdf = danfe_pdf.getvalue()
        danfe_pdf.close()

        if self.salvar_arquivo:
            nome_arq = self.caminho + self.NFe.chave + '.pdf'
            self.danfe.generate_by(PDFGenerator, filename=nome_arq)



# -*- coding: utf-8 -*-

from pysped.xml_sped import *
from pysped.cte.leiaute import ESQUEMA_ATUAL_VERSAO_300 as ESQUEMA_ATUAL
import os

from .cte_300 import CTe

DIRNAME = os.path.dirname(__file__)

class EnviCTe(XMLNFe):
    def __init__(self):
        super(EnviCTe, self).__init__()
        self.versao = TagDecimal(nome='enviCTe', codigo='AP01', propriedade='versao', namespace=NAMESPACE_CTE, valor='3.00', raiz='/')
        self.idLote  = TagInteiro(nome='idLote' , codigo='AP03', tamanho=[1, 15, 1], raiz='//enviCTe')
        self.CTe  = []
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'enviCTe_v3.00.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.idLote.xml

        for n in self.CTe:
            xml += tira_abertura(n.xml)

        xml += '</enviCTe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml    = arquivo
            self.idLote.xml    = arquivo
            self.CTe = self.le_grupo('//enviLote/CTe', CTe)

    xml = property(get_xml, set_xml)


class InfRec(XMLNFe):
    def __init__(self):
        super(InfRec, self).__init__()
        self.nRec     = TagCaracter(nome='nRec'     , codigo='AR08', tamanho=[1, 15, 1], raiz='//retEnviCte/infRec', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.dhRecbto = TagDataHora(nome='dhRecbto', codigo='AR09'                    , raiz='//retEnviCte/infRec', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.tMed     = TagInteiro(nome='tMed'     , codigo='AR10', tamanho=[1,  4, 1], raiz='//retEnviCte/infRec', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)

    def get_xml(self):
        if not self.nRec.valor:
            return ''

        xml = XMLNFe.get_xml(self)
        xml += '<infRec>'
        xml += self.nRec.xml
        xml += self.dhRecbto.xml
        xml += self.tMed.xml
        xml += '</infRec>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nRec.xml     = arquivo
            self.dhRecbto.xml = arquivo
            self.tMed.xml     = arquivo

    xml = property(get_xml, set_xml)


class RetEnviCTe(XMLNFe):
    def __init__(self):
        super(RetEnviCTe, self).__init__()
        self.versao   = TagDecimal(nome='retEnviCte', codigo='AR02' , propriedade='versao', namespace=NAMESPACE_CTE, valor='3.00', raiz='/')
        self.tpAmb    = TagInteiro(nome='tpAmb'     , codigo='AR03' , tamanho=[1,   1, 1], raiz='//retEnviCte')
        self.cUF      = TagCaracter(nome='cUF'      , codigo='AR03a', tamanho=[2,   2, 2], raiz='//retEnviCte')
        self.verAplic = TagCaracter(nome='verAplic' , codigo='AR04' , tamanho=[1,  20]   , raiz='//retEnviCte')
        self.cStat    = TagCaracter(nome='cStat'    , codigo='AR05' , tamanho=[1,   3]   , raiz='//retEnviCte')
        self.xMotivo  = TagCaracter(nome='xMotivo'  , codigo='AR06' , tamanho=[1, 255]   , raiz='//retEnviCte')
        self.infRec   = InfRec()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema/', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retEnviCTe_v3.00.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.infRec.xml
        xml += '</retEnviCte>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml   = arquivo
            self.tpAmb.xml    = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml    = arquivo
            self.xMotivo.xml  = arquivo
            self.cUF.xml      = arquivo
            self.infRec.xml   = arquivo

    xml = property(get_xml, set_xml)



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import (ABERTURA, NAMESPACE_NFE, Signature, TagCaracter,
                             TagDataHora, TagDecimal, TagInteiro, XMLNFe)
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_3 as ESQUEMA_ATUAL
from pysped.nfe.leiaute.soap_200 import NFeDadosMsg
import os


DIRNAME = os.path.dirname(__file__)


class DistNSU(XMLNFe):
    def __init__(self):
        super(DistNSU, self).__init__()
        self.ultNSU = TagCaracter(nome='ultNSU', tamanho=[1, 15], raiz='//distDFeInt/distNSU', valor='0')

    def get_xml(self):
        if not self.ultNSU.valor:
            return ''

        xml = XMLNFe.get_xml(self)
        xml += '<distNSU>'
        xml += self.ultNSU.xml
        xml += '</distNSU>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.ultNSU.xml = arquivo

    xml = property(get_xml, set_xml)


class ConsNSU(XMLNFe):
    def __init__(self):
        super(ConsNSU, self).__init__()
        self.NSU = TagCaracter(nome='NSU', tamanho=[1, 15], raiz='//distDFeInt/consNSU', valor='')

    def get_xml(self):
        if not self.NSU.valor:
            return ''

        xml = XMLNFe.get_xml(self)
        xml += '<consNSU>'
        xml += self.NSU.xml
        xml += '</consNSU>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml = arquivo

    xml = property(get_xml, set_xml)


class DistDFeInt(XMLNFe):
    def __init__(self):
        super(DistDFeInt, self).__init__()
        self.versao   = TagDecimal(nome='distDFeInt', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.00', raiz='/')
        self.tpAmb    = TagInteiro(nome='tpAmb'     , tamanho=[1, 1, 1], raiz='//distDFeInt', valor=2)
        self.cUFAutor = TagInteiro(nome='cUFAutor'  , tamanho=[2, 2, 2], raiz='//distDFeInt')
        self.CNPJ     = TagCaracter(nome='CNPJ'     , tamanho=[14, 14] , raiz='//distDFeInt', obrigatorio=False)
        self.CPF      = TagCaracter(nome='CPF'      , tamanho=[11, 11] , raiz='//distDFeInt', obrigatorio=False)
        self.distNSU  = DistNSU()
        self.consNSU  = ConsNSU()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'distDFeInt_v1.00.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.cUFAutor.xml

        if self.CNPJ.valor:
            xml += self.CNPJ.xml
        else:
            xml += self.CPF.xml

        if self.consNSU.NSU.valor is not None and self.consNSU.NSU.valor != '':
            xml += self.consNSU.xml
        else:
            xml += self.distNSU.xml

        xml += '</distDFeInt>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml     = arquivo
            self.tpAmb.xml  = arquivo
            self.cUFAutor.xml    = arquivo
            self.CNPJ.xml   = arquivo
            self.CPF.xml   = arquivo
            self.distNSU.xml    = arquivo
            self.consNSU.xml    = arquivo

    xml = property(get_xml, set_xml)


class DocZip(XMLNFe):
    def __init__(self):
        super(DocZip, self).__init__()
        self.NSU    = TagCaracter(nome='docZip', propriedade='NSU'   , namespace=NAMESPACE_NFE, raiz='//docZip')
        self.schema = TagCaracter(nome='docZip', propriedade='schema', namespace=NAMESPACE_NFE, raiz='//docZip')
        self.base64Binary = TagCaracter(nome='', namespace=NAMESPACE_NFE, raiz='//docZip')

    def get_xml(self):
        xml = XMLNFe.get_xml(self)

        xml += self.NSU.xml
        xml += self.schema.xml
        xml += self.base64Binary.xml

        xml += '</docZip>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.NSU.xml    = arquivo
            self.schema.xml = arquivo
            self.base64Binary.xml = arquivo

    xml = property(get_xml, set_xml)


class LoteDistDFeInt(XMLNFe):
    def __init__(self):
        super(LoteDistDFeInt, self).__init__()
        self.docZip = []

    def get_xml(self):
        xml = XMLNFe.get_xml(self)

        if len(self.docZip) == 0:
            return xml

        xml += '<loteDistDFeInt>'

        for d in self.docZip:
            xml += d.xml

        xml += '</loteDistDFeInt>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            #
            # Técnica para leitura de tags múltiplas
            # As classes dessas tags, e suas filhas, devem ser
            # "reenraizadas" (propriedade raiz) para poderem ser
            # lidas corretamente
            #
            self.docZip = self.le_grupo('//loteDistDFeInt/docZip', DocZip)

    xml = property(get_xml, set_xml)


class RetDistDFeInt(XMLNFe):
    def __init__(self):
        super(RetDistDFeInt, self).__init__()
        self.versao   = TagDecimal(nome='retDistDFeInt', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.00', raiz='/')
        self.tpAmb    = TagInteiro(nome='tpAmb'        , tamanho=[1, 1, 1], raiz='//retDistDFeInt', valor=2)
        self.verAplic = TagCaracter(nome='verAplic'    , tamanho=[1, 20]  , raiz='//retDistDFeInt')
        self.cStat    = TagCaracter(nome='cStat'       , tamanho=[3, 3, 3], raiz='//retDistDFeInt')
        self.xMotivo  = TagCaracter(nome='xMotivo'     , tamanho=[1, 255] , raiz='//retDistDFeInt')
        self.dhResp   = TagDataHora(nome='dhResp'                      , raiz='//retDistDFeInt')
        self.ultNSU   = TagCaracter(nome='ultNSU'      , tamanho=[1, 15]  , raiz='//retDistDFeInt', obrigatorio=False)
        self.maxNSU   = TagCaracter(nome='maxNSU'      , tamanho=[1, 15]  , raiz='//retDistDFeInt', obrigatorio=False)
        self.loteDistDFeInt = LoteDistDFeInt()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retDistDFeInt_v1.00.xsd'

        self.chave = ''

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.dhResp.xml
        xml += self.ultNSU.xml
        xml += self.maxNSU.xml
        xml += self.loteDistDFeInt.xml
        xml += '</retDistDFeInt>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpAmb.xml    = arquivo
            self.verAplic.xml = arquivo
            self.cStat.xml    = arquivo
            self.xMotivo.xml  = arquivo
            self.dhResp.xml  = arquivo
            self.ultNSU.xml  = arquivo
            self.maxNSU.xml  = arquivo
            self.loteDistDFeInt.xml = arquivo

    xml = property(get_xml, set_xml)


class SOAPEnvio(XMLNFe):
    def __init__(self):
        super(SOAPEnvio, self).__init__()
        self.webservice = ''
        self.metodo = ''
        self.cUF    = None
        self.envio  = None
        self.nfeDadosMsg = NFeDadosMsg()
        self._header = {b'content-type': b'application/soap+xml; charset=utf-8'}
        self.soap_action_webservice_e_metodo = False

    def get_xml(self):
        self.nfeDadosMsg.webservice = self.webservice
        self.nfeDadosMsg.dados = self.envio

        self._header[b'content-type'] = b'application/soap+xml; charset=utf-8;'  #  ' action="http://www.portalfiscal.inf.br/nfe/wsdl/' + self.webservice.encode('utf-8') + b'/' + self.metodo.encode('utf-8') + b'"'

        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += '<soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">'
        xml +=     '<soap12:Body>'
        xml +=       '<nfeDistDFeInteresse xmlns="http://www.portalfiscal.inf.br/nfe/wsdl/NFeDistribuicaoDFe">'
        xml +=             self.nfeDadosMsg.xml
        xml +=       '</nfeDistDFeInteresse>'
        xml +=     '</soap12:Body>'
        xml += '</soap12:Envelope>'
        return xml

    def set_xml(self):
        pass

    xml = property(get_xml, set_xml)

    def get_header(self):
        header = self._header
        return header

    header = property(get_header)


class SOAPRetorno(XMLNFe):
    def __init__(self):
        super(SOAPRetorno, self).__init__()
        self.webservice = ''
        self.metodo = ''
        self.resposta = None

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += '<soap:Envelope xmlns:soap="http://www.w3.org/2003/05/soap-envelope">'
        xml +=     '<soap:Header>'
        xml +=         '<nfeCabecMsg xmlns="http://www.portalfiscal.inf.br/nfe/wsdl/' + self.webservice + '">'
        xml +=             self.nfeCabecMsg.xml
        xml +=         '</nfeCabecMsg>'
        xml +=     '</soap:Header>'
        xml +=     '<soap:Body>'
        xml +=         '<nfeDistDFeInteresseResponse xmlns="http://www.portalfiscal.inf.br/nfe/wsdl/NFeDistribuicaoDFe">'
        xml +=             '<' + self.metodo + 'Result>'
        xml +=                 self.resposta.xml
        xml +=             '</' + self.metodo + 'Result>'
        xml +=     '</soap:Body>'
        xml += '</soap:Envelope>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.resposta.xml = arquivo

    xml = property(get_xml, set_xml)



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Affero General Public License,
# publicada pela Free Software Foundation, em sua versão 3 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Affero General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Affero General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import *
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_3 as ESQUEMA_ATUAL
from pysped.nfe.leiaute import inutnfe_200
import os


DIRNAME = os.path.dirname(__file__)


class InfInutEnviado(inutnfe_200.InfInutEnviado):
    def __init__(self):
        super(InfInutEnviado, self).__init__()
        self.Id     = TagCaracter(nome='infInut', codigo='DP03', tamanho=[43, 43] , raiz='//inutNFe', propriedade='Id')


class InutNFe(inutnfe_200.InutNFe):
    def __init__(self):
        super(InutNFe, self).__init__()
        self.versao  = TagDecimal(nome='inutNFe', codigo='DP01', propriedade='versao', namespace=NAMESPACE_NFE, valor='3.10', raiz='/')
        self.infInut = InfInutEnviado()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'inutNFe_v3.10.xsd'


class InfInutRecebido(inutnfe_200.InfInutRecebido):
    def __init__(self):
        super(InfInutRecebido, self).__init__()
        self.dhRecbto = TagDataHoraUTC(nome='dhRecbto', codigo='DR16', raiz='//retInutNFe/infInut', obrigatorio=False)


class RetInutNFe(inutnfe_200.RetInutNFe):
    def __init__(self):
        super(RetInutNFe, self).__init__()
        self.versao = TagDecimal(nome='retInutNFe', codigo='DR01', propriedade='versao', namespace=NAMESPACE_NFE, valor='3.10', raiz='/')
        self.infInut = InfInutRecebido()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retInutNFe_v3.10.xsd'


class ProcInutNFe(inutnfe_200.ProcInutNFe):
    def __init__(self):
        super(ProcInutNFe, self).__init__()
        #
        # Atenção --- a tag ProcInutNFe tem que começar com letra maiúscula, para
        # poder validar no XSD. Os outros arquivos proc, procCancNFe, e procNFe
        # começam com minúscula mesmo
        #
        self.versao = TagDecimal(nome='ProcInutNFe', propriedade='versao', namespace=NAMESPACE_NFE, valor='3.10', raiz='/')
        self.inutNFe = InutNFe()
        self.retInutNFe = RetInutNFe()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'procInutNFe_v3.10.xsd'



# -*- coding: utf-8 -*-

WS_CTE_RECEPCAO  = 0
WS_CTE_RET_RECEPCAO  = 1
WS_CTE_INUTILIZACAO  = 2
WS_CTE_CONSULTA  = 3
WS_CTE_STATUS_SERVICO = 4
WS_CTE_EVENTO  = 5
WS_CTE_CONSULTA_CADASTRO  = 6

CTE_AMBIENTE_PRODUCAO    = 1
CTE_AMBIENTE_HOMOLOGACAO = 2


from setuptools import setup

setup(
    name = "PySPED",
    version = "1.0.1",
    author = "Aristides Caldeira",
    author_email = 'aristides.caldeira@tauga.com.br',
    test_suite='tests',
    keywords = ['nfe', 'nfse', 'cte', 'sped', 'edf', 'ecd'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Plugins',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages = [
        'pysped',
        'pysped.nfe',
        'pysped.nfe.leiaute',
        'pysped.nfe.danfe',
        'pysped.nfe.manual_300',
        'pysped.nfe.manual_401',
        'pysped.cte',
        'pysped.cte.leiaute',
        'pysped.cte.dacte',
        'pysped.efd',
        #'pysped.nfse',
        'pysped.xml_sped',
        'pysped.ecd',
        'pysped.nf_paulista',
        'pysped.relato_sped',
        'pysped.exemplos',
    ],
    package_data = {
        'pysped.nfe.danfe': ['fonts/*', '*.odt'],
        'pysped.relato_sped': ['fonts/*'],
        'pysped.nfe.leiaute': ['schema/*/*'],
        'pysped.cte.leiaute': ['schema/*/*'],
        'pysped.xml_sped': ['cadeia-certificadora/*/*']
    },
    url = 'https://github.com/aricaldeira/PySPED',
    license = 'LGPL-v2.1+',
    description = 'PySPED is a library to implement all requirements of the Brazilian Public System of Digital Bookkeeping',
    long_description = open('README.rst').read(),
    requires=[
        'lxml(>=3.7.3)',
        'xmlsec(>=1.0.7)',
        'Geraldo(>=0.4.16)',
        'qrcode(>=5.3)',
        'py3o.template(>=0.9.11)',
        'sh(>=1.12.9)'
    ],
    tests_require=[
        'pyflakes(>=0.6.1)',
    ],
)



# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals


WS_NFE_ENVIO_LOTE = 0
WS_NFE_CONSULTA_RECIBO = 1

#
# Novos webservices na versão 3.10, para as mesmas
# funções dos antigos
#
WS_NFE_AUTORIZACAO = WS_NFE_ENVIO_LOTE
WS_NFE_CONSULTA_AUTORIZACAO = WS_NFE_CONSULTA_RECIBO

WS_NFE_CANCELAMENTO = 2
WS_NFE_INUTILIZACAO = 3
WS_NFE_CONSULTA = 4
WS_NFE_SITUACAO = 5
WS_NFE_CONSULTA_CADASTRO = 6

WS_DPEC_RECEPCAO = 7
WS_DPEC_CONSULTA = 8

WS_NFE_RECEPCAO_EVENTO = 9
WS_NFE_DOWNLOAD = 10
WS_NFE_CONSULTA_DESTINADAS = 11
WS_DFE_DISTRIBUICAO = 12

NFE_AMBIENTE_PRODUCAO = 1
NFE_AMBIENTE_HOMOLOGACAO = 2

UF_CODIGO = {
    'AC': 12,
    'AL': 27,
    'AM': 13,
    'AP': 16,
    'BA': 29,
    'CE': 23,
    'DF': 53,
    'ES': 32,
    'GO': 52,
    'MA': 21,
    'MG': 31,
    'MS': 50,
    'MT': 51,
    'PA': 15,
    'PB': 25,
    'PE': 26,
    'PI': 22,
    'PR': 41,
    'RJ': 33,
    'RN': 24,
    'RO': 11,
    'RR': 14,
    'RS': 43,
    'SC': 42,
    'SE': 28,
    'SP': 35,
    'TO': 17,
    'SUFRAMA': 90, # Código especial para eventos
    'RFB': 91, # Código especial para eventos
}

CODIGO_UF = {
    12: 'AC',
    27: 'AL',
    13: 'AM',
    16: 'AP',
    29: 'BA',
    23: 'CE',
    53: 'DF',
    32: 'ES',
    52: 'GO',
    21: 'MA',
    31: 'MG',
    50: 'MS',
    51: 'MT',
    15: 'PA',
    25: 'PB',
    26: 'PE',
    22: 'PI',
    41: 'PR',
    33: 'RJ',
    24: 'RN',
    11: 'RO',
    14: 'RR',
    43: 'RS',
    42: 'SC',
    28: 'SE',
    35: 'SP',
    17: 'TO',
    90: 'SUFRAMA', # Código especial para eventos
    91: 'RFB', # Código especial para eventos
}




# -*- coding: utf-8 -*-
#
# PySPED - Python libraries to deal with Brazil's SPED Project
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira at tauga.com.br>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Library General Public License as
# published by the Free Software Foundation, either version 2.1 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU Library General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# PySPED - Bibliotecas Python para o
#          SPED - Sistema Público de Escrituração Digital
#
# Copyright (C) 2010-2012
# Copyright (C) Aristides Caldeira <aristides.caldeira arroba tauga.com.br>
#
# Este programa é um software livre: você pode redistribuir e/ou modificar
# este programa sob os termos da licença GNU Library General Public License,
# publicada pela Free Software Foundation, em sua versão 2.1 ou, de acordo
# com sua opção, qualquer versão posterior.
#
# Este programa é distribuido na esperança de que venha a ser útil,
# porém SEM QUAISQUER GARANTIAS, nem mesmo a garantia implícita de
# COMERCIABILIDADE ou ADEQUAÇÃO A UMA FINALIDADE ESPECÍFICA. Veja a
# GNU Library General Public License para mais detalhes.
#
# Você deve ter recebido uma cópia da GNU Library General Public License
# juntamente com este programa. Caso esse não seja o caso, acesse:
# <http://www.gnu.org/licenses/>
#

from __future__ import division, print_function, unicode_literals

from pysped.xml_sped import (ABERTURA, NAMESPACE_NFE, Signature, TagCaracter,
                             TagDataHora, TagDecimal, TagInteiro, XMLNFe)
from pysped.nfe.leiaute import ESQUEMA_ATUAL_VERSAO_1 as ESQUEMA_ATUAL
import os


DIRNAME = os.path.dirname(__file__)


class ConsSitNFe(XMLNFe):
    def __init__(self):
        super(ConsSitNFe, self).__init__()
        self.versao = TagDecimal(nome='consSitNFe', codigo='EP01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.07', raiz='/')
        self.tpAmb  = TagInteiro(nome='tpAmb'     , codigo='EP03', tamanho=[ 1,  1, 1], raiz='//consSitNFe', valor=2)
        self.xServ  = TagCaracter(nome='xServ'    , codigo='EP04', tamanho=[ 9,  9]   , raiz='//consSitNFe', valor='CONSULTAR')
        self.chNFe  = TagCaracter(nome='chNFe'    , codigo='EP05', tamanho=[44, 44]   , raiz='//consSitNFe')
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'consSitNFe_v1.07.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.tpAmb.xml
        xml += self.xServ.xml
        xml += self.chNFe.xml
        xml += '</consSitNFe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml = arquivo
            self.tpAmb.xml  = arquivo
            self.xServ.xml  = arquivo
            self.chNFe.xml  = arquivo

    xml = property(get_xml, set_xml)


class InfProt(XMLNFe):
    '''Atenção!!!

    Este grupo infProt é DIFERENTE do infProt do retorno do recibo do lote

    Colocar esse infProt dentro do arquivo procNFe vai fazer com que o procNFe gerado
    seja INVALIDADO pelo XSD!!!

    Para transportar os valores desta infProt para a infProt do procNFe, é preciso usar

    procNFe.protNFe.infProt.xml = este_infProt.xml

    '''
    def __init__(self):
        super(InfProt, self).__init__()
        self.Id        = TagCaracter(nome='infProt' , codigo='ER04' , propriedade='Id'  , raiz='/'        , obrigatorio=False)
        self.tpAmb     = TagInteiro(nome='tpAmb'    , codigo='ER05' , tamanho=[1,   1, 1], raiz='//infProt')
        self.verAplic  = TagCaracter(nome='verAplic', codigo='ER06' , tamanho=[1,  20]   , raiz='//infProt')
        self.cStat     = TagCaracter(nome='cStat'   , codigo='ER07' , tamanho=[1,   3]   , raiz='//infProt')
        self.xMotivo   = TagCaracter(nome='xMotivo' , codigo='ER08' , tamanho=[1, 2000]  , raiz='//infProt')
        self.cUF       = TagInteiro(nome='cUF'      , codigo='ER08a', tamanho=[2,   2, 2], raiz='//infProt')
        self.chNFe     = TagCaracter(nome='chNFe'   , codigo='ER09' , tamanho=[44, 44]   , raiz='//infProt', obrigatorio=False)
        self.dhRecbto  = TagDataHora(nome='dhRecbto', codigo='ER10'                      , raiz='//infProt', obrigatorio=False)
        self.nProt     = TagCaracter(nome='nProt'   , codigo='ER11' , tamanho=[15, 15]   , raiz='//infProt', obrigatorio=False)
        self.digVal    = TagCaracter(nome='digVal'  , codigo='ER12' , tamanho=[28, 28]   , raiz='//infProt', obrigatorio=False)

    def get_xml(self):
        xml = XMLNFe.get_xml(self)

        if self.Id.valor:
            xml += self.Id.xml
        else:
            xml += '<infProt>'

        xml += self.tpAmb.xml
        xml += self.verAplic.xml
        xml += self.cStat.xml
        xml += self.xMotivo.xml
        xml += self.cUF.xml
        xml += self.chNFe.xml
        xml += self.dhRecbto.xml
        xml += self.nProt.xml
        xml += self.digVal.xml
        xml += '</infProt>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.Id.xml        = arquivo
            self.tpAmb.xml     = arquivo
            self.verAplic.xml  = arquivo
            self.cStat.xml     = arquivo
            self.xMotivo.xml   = arquivo
            self.cUF.xml       = arquivo
            self.chNFe.xml     = arquivo
            self.dhRecbto.xml  = arquivo
            self.nProt.xml     = arquivo
            self.digVal.xml    = arquivo

    xml = property(get_xml, set_xml)


class RetConsSitNFe(XMLNFe):
    def __init__(self):
        super(RetConsSitNFe, self).__init__()
        self.versao    = TagDecimal(nome='retConsSitNFe', codigo='ER01', propriedade='versao', namespace=NAMESPACE_NFE, valor='1.07', raiz='/')
        self.infProt   = InfProt()
        self.Signature = Signature()
        self.caminho_esquema = os.path.join(DIRNAME, 'schema', ESQUEMA_ATUAL + '/')
        self.arquivo_esquema = 'retConsSitNFe_v1.07.xsd'

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += ABERTURA
        xml += self.versao.xml
        xml += self.infProt.xml

        if len(self.Signature.URI) and (self.Signature.URI.strip() != '#'):
            xml += self.Signature.xml

        xml += '</retConsSitNFe>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.versao.xml  = arquivo
            #
            # o grupo infProt é usado também no webservice de consulta do recibo de lote de NF-e
            # por isso, a raiz dele não pode ser assumida como sendo sempre o grupo
            # retConsSitNFe
            #
            self.infProt.xml   = self._le_noh('//retConsSitNFe/infProt')
            self.Signature.xml = self._le_noh('//retConsSitNFe/sig:Signature')

    xml = property(get_xml, set_xml)



# -*- coding: utf-8 -*-

from pysped.xml_sped import *
from pysped.cte.leiaute import ESQUEMA_ATUAL_VERSAO_300 as ESQUEMA_ATUAL
import os

DIRNAME = os.path.dirname(__file__)

class EmiOcc(XMLNFe):
    def __init__(self):
        super(EmiOcc, self).__init__()
        self.CNPJ   = TagCaracter(nome='CNPJ', tamanho=[ 0, 14], raiz='//emiOcc', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.cInt   = TagCaracter(nome='cInt', tamanho=[ 1, 10], raiz='//emiOcc', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.IE     = TagCaracter(nome='IE', tamanho=[ 2, 14], raiz='//emiOcc', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.UF     = TagCaracter(nome='UF', tamanho=[ 2,  2]   , raiz='//emiOcc', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.fone   = TagInteiro(nome='fone', tamanho=[ 6, 14], raiz='//emiOcc', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        
    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '<emiOcc>'
        xml += self.CNPJ.xml
        xml += self.cInt.xml
        xml += self.IE.xml
        xml += self.UF.xml
        xml += self.fone.xml
        xml += '</emiOcc>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml   = arquivo
            self.cInt.xml   = arquivo
            self.IE.xml     = arquivo
            self.UF.xml     = arquivo
            self.fone.xml   = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Occ(XMLNFe):
    def __init__(self):
        super(Occ, self).__init__()
        self.serie = TagCaracter(nome='serie' , tamanho=[ 8, 8, 8], raiz='//occ', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.nOcc  = TagInteiro(nome='nOcc' , tamanho=[ 1,  6], raiz='//occ', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.dEmi  = TagData(nome='dEmi', raiz='//occ', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.emiOcc = EmiOcc()
        
    def get_xml(self):
        if not (self.nOcc.valor or self.dEmi.valor or self.emiOcc is not None):
            return ''
            
        xml = XMLNFe.get_xml(self)
        xml += u'<occ>'
        xml += self.serie.xml
        xml += self.nOcc.xml
        xml += self.dEmi.xml
        xml += self.emiOcc.xml
        xml += '</occ>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.serie.xml  = arquivo
            self.nOcc.xml   = arquivo
            self.dEmi.xml   = arquivo
            self.emiOcc.xml = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Rodo(XMLNFe):
    def __init__(self):
        super(Rodo, self).__init__()
        self.RNTRC = TagCaracter(nome='RNTRC' , tamanho=[ 8, 8, 8], raiz='//rodo', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.occ = []

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<rodo>'
        xml += self.RNTRC.xml
        for o in self.occ:
            xml += o.xml
            
        xml += '</rodo>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.RNTRC.xml = arquivo
            self.occ = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/rodo/occ', Occ, sigla_ns='cte')
            
    xml = property(get_xml, set_xml)
    

class InfTotAP(XMLNFe):
    def __init__(self):
        super(InfTotAP, self).__init__()
        self.qTotProd = TagCaracter(nome='qTotProd' , tamanho=[ 1, 1, 1], raiz='//infTotAP', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.uniAP = TagCaracter(nome='uniAP' , tamanho=[ 1, 4], raiz='//infTotAP', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        
    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<infTotAP>'
        xml += self.qTotProd.xml
        xml += self.uniAP.xml
        xml += '</infTotAP>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.qTotProd.xml = arquivo
            self.uniAP.xml    = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Peri(XMLNFe):
    def __init__(self):
        super(Peri, self).__init__()
        self.nONU = TagCaracter(nome='nONU' , tamanho=[ 4, 4, 4], raiz='//peri', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.qTotEmb = TagCaracter(nome='qTotEmb' , tamanho=[ 1, 20], raiz='//peri', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.infTotAP = InfTotAP()
        
    def get_xml(self):
        if not (self.nONU.valor or self.qTotEmb.valor or self.infTotAP is not None):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<peri>'
        xml += self.nONU.xml
        xml += self.qTotEmb.xml
        xml += self.infTotAP.xml
        xml += '</peri>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nONU.xml      = arquivo
            self.qTotEmb.xml   = arquivo
            self.infTotAP.xml  = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Tarifa(XMLNFe):
    def __init__(self):
        super(Tarifa, self).__init__()
        self.CL = TagCaracter(nome='CL' , tamanho=[ 1, 1, 1], raiz='//tarifa', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.cTar = TagCaracter(nome='cTar' , tamanho=[ 1, 4], raiz='//tarifa', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.vTar = TagDecimal(nome='vTar', tamanho=[1, 13, 1], decimais=[0, 2, 2], raiz='//tarifa', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        
    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<tarifa>'
        xml += self.CL.xml
        xml += self.cTar.xml
        xml += self.vTar.xml
        xml += '</tarifa>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CL.xml     = arquivo
            self.cTar.xml   = arquivo
            self.vTar.xml   = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class TagCInfManu(TagCaracter):
    def __init__(self, *args, **kwargs):
        super(TagCInfManu, self).__init__(*args, **kwargs)
        self.nome = 'cInfManu'
        self.tamanho = [2, 2]
        self.raiz = '//natCarga'
        
    
class NatCarga(XMLNFe):
    def __init__(self):
        super(NatCarga, self).__init__()
        self.xDime = TagCaracter(nome='xDime' , tamanho=[ 5, 14], raiz='//natCarga', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.cInfManu = []
        
    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<natCarga>'
        xml += self.xDime.xml
        for c in self.cInfManu:
            xml += c.xml
            
        xml += '</natCarga>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.xDime.xml = arquivo
            self.cInfManu = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/aereo/natCarga/cInfManu', TagCInfManu, sigla_ns='cte')
            
    xml = property(get_xml, set_xml)
    
    
class Aereo(XMLNFe):
    def __init__(self):
        super(Aereo, self).__init__()
        self.nMinu = TagInteiro(nome='nMinu' , tamanho=[ 9, 9, 9], raiz='//aereo', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.nOCA = TagInteiro(nome='nOCA' , tamanho=[ 11, 11, 11], raiz='//aereo', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.dPrevAereo = TagData(nome='dPrevAereo' , raiz='//aereo', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.natCarga = NatCarga()
        self.tarifa   = Tarifa()
        self.peri     = []

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<aereo>'
        xml += self.nMinu.xml
        xml += self.nOCA.xml
        xml += self.dPrevAereo.xml
        xml += self.natCarga.xml
        xml += self.tarifa.xml
        for p in self.peri:
            xml += p.xml
            
        xml += '</aereo>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nMinu.xml      = arquivo
            self.nOCA.xml       = arquivo
            self.dPrevAereo.xml = arquivo
            self.natCarga.xml   = arquivo
            self.tarifa.xml     = arquivo
            self.peri = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/aereo/peri', Peri, sigla_ns='cte')
            
    xml = property(get_xml, set_xml)
    

class InfNFeAquav(XMLNFe):
    def __init__(self):
        super(InfNFeAquav, self).__init__()
        self.chave = TagCaracter(nome='chave', tamanho=[ 44, 44], raiz='//CTe/infCte/infCTeNorm/infModal/aquav/detCont/infDoc/infNFe', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.unidRat = TagDecimal(nome='unidRat', tamanho=[1, 3, 1], decimais=[0, 2, 2], raiz='//CTe/infCte/infCTeNorm/infModal/aquav/detCont/infDoc/infNFe', obrigatorio=False)
        
    def get_xml(self):
        if not (self.chave.valor):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<infNF>'
        xml += self.chave.xml            
        xml += self.unidRat.xml            
        xml += '</infNF>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.chave.xml = arquivo
            self.unidRat.xml = arquivo
            
    xml = property(get_xml, set_xml)
    

class InfNFAquav(XMLNFe):
    def __init__(self):
        super(InfNFAquav, self).__init__()
        self.serie = TagCaracter(nome='serie', tamanho=[ 1, 3], raiz='//CTe/infCte/infCTeNorm/infModal/aquav/detCont/infDoc/infNF', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.nDoc = TagCaracter(nome='nDoc', tamanho=[ 1, 20], raiz='//CTe/infCte/infCTeNorm/infModal/aquav/detCont/infDoc/infNF', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.unidRat = TagDecimal(nome='unidRat', tamanho=[1, 3, 1], decimais=[0, 2, 2], raiz='//CTe/infCte/infCTeNorm/infModal/aquav/detCont/infDoc/infNF', obrigatorio=False)
        
    def get_xml(self):
        if not (self.serie.valor or self.nDoc.valor):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<infNF>'
        xml += self.serie.xml            
        xml += self.nDoc.xml            
        xml += self.unidRat.xml            
        xml += '</infNF>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.serie.xml = arquivo
            self.nDoc.xml = arquivo
            self.unidRat.xml = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class InfDocAquav(XMLNFe):
    def __init__(self):
        super(InfDocAquav, self).__init__()
        self.infNF  = []
        self.infNFe = []
        
    def get_xml(self):
        if (len(self.infNF)==0 and len(self.infNFe)==0):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<infDoc>'
        for inf in self.infNF:
            xml += inf.xml
        for infe in self.infNFe:
            xml += infe.xml
        xml += '</infDoc>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infNF = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/aquav/detCont/infDoc/infNF', InfNFAquav, sigla_ns='cte')
            self.infNFe = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/aquav/detCont/infDoc/infNFe', InfNFeAquav, sigla_ns='cte')
            
    xml = property(get_xml, set_xml)
    
    
class Lacre(XMLNFe):
    def __init__(self):
        super(Lacre, self).__init__()
        self.nLacre = TagCaracter(nome='nLacre' , tamanho=[ 1, 20], raiz='//lacre', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        
    def get_xml(self):
        if not (self.nLacre.valor):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<lacre>'
        xml += self.nLacre.xml            
        xml += '</lacre>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nLacre.xml = arquivo
            
    xml = property(get_xml, set_xml)

    
class DetCont(XMLNFe):
    def __init__(self):
        super(DetCont, self).__init__()
        self.nCont = TagCaracter(nome='xBalsa' , tamanho=[ 1, 20], raiz='//detCont', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.lacre = []
        ##Evitar conflito de nome com InfDoc
        self.infDoc = InfDocAquav()
        
    def get_xml(self):
        if not (self.nCont.valor):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<detCont>'
        xml += self.nCont.xml            
        for l in self.lacre:
            xml += l.xml
            
        xml += self.infDoc.xml            
        xml += '</detCont>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.nCont.xml = arquivo
            self.infDoc.xml = arquivo
            self.lacre = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/aquav/detCont/lacre', Lacre, sigla_ns='cte')
            
    xml = property(get_xml, set_xml)

    
class Balsa(XMLNFe):
    def __init__(self):
        super(Balsa, self).__init__()
        self.xBalsa = TagCaracter(nome='xBalsa' , tamanho=[ 1, 60], raiz='//balsa', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        
    def get_xml(self):
        if not (self.xBalsa.valor):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<balsa>'
        xml += self.xBalsa.xml            
        xml += '</balsa>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.xBalsa.xml = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Aquav(XMLNFe):
    def __init__(self):
        super(Aquav, self).__init__()
        self.vPrest = TagDecimal(nome='vPrest', tamanho=[1, 13, 1], decimais=[0, 2, 2], raiz='//aquav', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.vAFRMM = TagDecimal(nome='vAFRMM', tamanho=[1, 13, 1], decimais=[0, 2, 2], raiz='//aquav', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.xNavio = TagCaracter(nome='xNavio' , tamanho=[1, 60], raiz='//aquav', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.balsa   = []
        self.nViag  = TagInteiro(nome='nViag', tamanho=[1, 10], raiz='//aquav', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.direc  = TagCaracter(nome='direc', tamanho=[1, 1, 1], raiz='//aquav', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.irin  = TagCaracter(nome='irin', tamanho=[1, 10], raiz='//aquav', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.detCont = []

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<aquav>'
        xml += self.vPrest.xml
        xml += self.vAFRMM.xml
        xml += self.xNavio.xml
        for b in self.balsa:
            xml += b.xml
        for d in self.detCont:
            xml += d.xml
            
        xml += self.nViag.xml
        xml += self.direc.xml
        xml += self.irin.xml
        xml += '</aquav>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.vPrest.xml  = arquivo
            self.vAFRMM.xml  = arquivo
            self.xNavio.xml  = arquivo
            self.nViag.xml   = arquivo
            self.direc.xml   = arquivo
            self.irin.xml    = arquivo
            self.balsa = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/aquav/balsa', Balsa, sigla_ns='cte')
            self.detCont = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/aquav/detCont', DetCont, sigla_ns='cte')
            
    xml = property(get_xml, set_xml)

    
class EnderFerro(XMLNFe):
    def __init__(self):
        super(EnderFerro, self).__init__()
        self.xLgr    = TagCaracter(nome='xLgr'   , tamanho=[ 2, 255]  , raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.nro     = TagCaracter(nome='nro'    , tamanho=[ 1, 60]   , raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.xCpl    = TagCaracter(nome='xCpl'   , tamanho=[ 1, 60]   , raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.xBairro = TagCaracter(nome='xBairro', tamanho=[ 2, 60]   , raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.cMun    = TagInteiro(nome='cMun'    , tamanho=[ 7,  7, 7], raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.xMun    = TagCaracter(nome='xMun'   , tamanho=[ 2, 60]   , raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.CEP     = TagCaracter(nome='CEP'    , tamanho=[ 8,  8, 8], raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.UF      = TagCaracter(nome='UF'     , tamanho=[ 2,  2]   , raiz='//enderFerro', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        
    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += '<enderFerro>'
        xml += self.xLgr.xml
        xml += self.nro.xml
        xml += self.xCpl.xml
        xml += self.xBairro.xml
        xml += self.cMun.xml
        xml += self.xMun.xml
        xml += self.CEP.xml
        xml += self.UF.xml
        xml += '</enderFerro>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.xLgr.xml    = arquivo
            self.nro.xml     = arquivo
            self.xCpl.xml    = arquivo
            self.xBairro.xml = arquivo
            self.cMun.xml    = arquivo
            self.xMun.xml    = arquivo
            self.CEP.xml     = arquivo
            self.UF.xml      = arquivo
            
    xml = property(get_xml, set_xml)    


class FerroEnv(XMLNFe):
    def __init__(self):
        super(FerroEnv, self).__init__()
        self.CNPJ   = TagCaracter(nome='CNPJ', tamanho=[ 0, 14], raiz='//ferroEnv', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.cInt   = TagCaracter(nome='cInt', tamanho=[ 1, 10], raiz='//ferroEnv', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.IE     = TagCaracter(nome='IE', tamanho=[ 2, 14], raiz='//ferroEnv', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.xNome     = TagCaracter(nome='xNome', tamanho=[ 2,  60]   , raiz='//ferroEnv', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.enderFerro   = EnderFerro()
        
    def get_xml(self):
        if not (self.CNPJ.valor):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<ferroEnv>'
        xml += self.CNPJ.xml            
        xml += self.cInt.xml            
        xml += self.IE.xml            
        xml += self.xNome.xml
        xml += self.enderFerro.xml
        xml += '</ferroEnv>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.CNPJ.xml   = arquivo
            self.cInt.xml   = arquivo
            self.IE.xml     = arquivo
            self.xNome.xml  = arquivo
            self.enderFerro.xml = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class TrafMut(XMLNFe):
    def __init__(self):
        super(TrafMut, self).__init__()
        self.respFat    = TagInteiro(nome='respFat' , tamanho=[ 1, 1, 1], raiz='//trafMut', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.ferrEmi    = TagInteiro(nome='ferrEmi' , tamanho=[ 1, 1, 1], raiz='//trafMut', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.vFrete     = TagDecimal(nome='vFrete', tamanho=[1, 13, 1], decimais=[0, 2, 2], raiz='//trafMut', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.chCTeFerroOrigem = TagCaracter(nome='chCTeFerroOrigem' , tamanho=[ 44, 44], raiz='//trafMut', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.ferroEnv = []
        
    def get_xml(self):
        if not (self.respFat.valor or self.ferrEmi.valor):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<trafMut>'
        xml += self.respFat.xml            
        xml += self.ferrEmi.xml            
        xml += self.vFrete.xml            
        xml += self.chCTeFerroOrigem.xml
        for f in self.ferroEnv:
            xml += f.xml
            
        xml += '</trafMut>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.respFat.xml = arquivo
            self.ferrEmi.xml = arquivo
            self.vFrete.xml = arquivo
            self.chCTeFerroOrigem.xml = arquivo
            self.ferroEnv = self.le_grupo('//CTe/infCte/infCTeNorm/infModal/ferrov/trafMut/ferroEnv', FerroEnv, sigla_ns='cte')
            
    xml = property(get_xml, set_xml)    
    
    
class Ferrov(XMLNFe):
    def __init__(self):
        super(Ferrov, self).__init__()
        self.tpTraf = TagInteiro(nome='tpTraf', tamanho=[1, 1, 1], raiz='//ferrov', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.trafMut = TrafMut()
        self.fluxo = TagCaracter(nome='fluxo', tamanho=[ 1, 10], raiz='//ferrov', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<ferrov>'
        xml += self.tpTraf.xml
        xml += self.trafMut.xml
        xml += self.fluxo.xml
        xml += '</ferrov>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.tpTraf.xml  = arquivo
            self.trafMut.xml  = arquivo
            self.fluxo.xml  = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Duto(XMLNFe):
    def __init__(self):
        super(Duto, self).__init__()
        self.vTar = TagDecimal(nome='vTar', tamanho=[1, 9, 1], decimais=[0, 6, 6], raiz='//duto', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)
        self.dIni = TagData(nome='dIni', raiz='//duto', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.dFim = TagData(nome='dFim', raiz='//duto', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<duto>'
        xml += self.vTar.xml
        xml += self.dIni.xml
        xml += self.dFim.xml
        xml += '</duto>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.vTar.xml = arquivo
            self.dIni.xml = arquivo
            self.dFim.xml = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class InfSeg(XMLNFe):
    def __init__(self):
        super(Seg, self).__init__()
        self.xSeg = TagCaracter(nome='xSeg', tamanho=[1, 30], raiz='//infSeg', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.CNPJ = TagCaracter(nome='CNPJ', tamanho=[ 0, 14], raiz='//infSeg', namespace=NAMESPACE_CTE, namespace_obrigatorio=False, obrigatorio=False)

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<infSeg>'
        xml += self.xSeg.xml
        xml += self.CNPJ.xml
        xml += '</infSeg>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.xSeg.xml = arquivo
            self.CNPJ.xml  = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Seg(XMLNFe):
    def __init__(self):
        super(Seg, self).__init__()
        self.infSeg = InfSeg()
        self.nApol = TagCaracter(nome='nApol', tamanho=[1, 20], raiz='//seg', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.nAver = TagCaracter(nome='nAver', tamanho=[1, 20], raiz='//seg', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)

    def get_xml(self):
        if not (self.nApol.valor or self.infSeg is not None):
            return ''
        xml = XMLNFe.get_xml(self)
        xml += u'<seg>'
        xml += self.infSeg.xml
        xml += self.nApol.xml
        xml += self.nAver.xml
        xml += '</seg>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.infSeg.xml = arquivo
            self.nApol.xml  = arquivo
            self.nAver.xml  = arquivo
            
    xml = property(get_xml, set_xml)
    
    
class Multimodal(XMLNFe):
    def __init__(self):
        super(Multimodal, self).__init__()
        self.COTM = TagCaracter(nome='COTM', tamanho=[1, 20], raiz='//multimodal', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.indNegociavel = TagInteiro(nome='indNegociavel', tamanho=[1, 1, 1], raiz='//multimodal', namespace=NAMESPACE_CTE, namespace_obrigatorio=False)
        self.seg = Seg()

    def get_xml(self):
        xml = XMLNFe.get_xml(self)
        xml += u'<multimodal>'
        xml += self.COTM.xml
        xml += self.indNegociavel.xml
        xml += self.seg.xml
        xml += '</multimodal>'
        return xml

    def set_xml(self, arquivo):
        if self._le_xml(arquivo):
            self.COTM.xml = arquivo
            self.indNegociavel.xml = arquivo
            self.seg.xml = arquivo
            
    xml = property(get_xml, set_xml)
    
